<?PHP
// Define a Windows or else Linux inc directory path
$_ENV['OS'] == 'Windows_NT' ? $incPath = 'inc\\' : $incPath = 'inc/';

require realpath($incPath."class.args.php");
define("SCRIPT_VERSION","BETA 0.0.1b");
define("RELEASE_DATE","24/11/2009");

function dieUsage($errorMsg){
  echo "\r\n";
  if ( $errorMsg != "" ) {
    echo "Error: $errorMsg\r\n\r\n";
  }
  echo "booklettibus ". SCRIPT_VERSION . " " . RELEASE_DATE . "\r\n\r\n";
  echo "Usage: booklettibus <input file> [<output file>]\r\n";
  echo "\r\n";
  echo "\tinput file\t: Original sla file of scribus 1.3.5 or greater (NO sla.gz)\r\n";
  echo "\toutput file\t: Optional output file name. If not given newfile have -booklet suffix\r\n";
  echo "\r\n";
  die(1);
}

function iseven($num){
    if (($num % 2) == 0) return true;
    return false;
}

function num2str($number){
  return preg_replace("/,/",".",  sprintf("%s", $number)); 
}

########################################################################
$args = new Args();

if ( (count($args->args) > 0) && (count($args->args) < 3) ) {
  //Do nothing
} else {
  dieUsage("Not enough parameters");
};

$inputFile=$args->args[0];
if (!file_exists($inputFile)) {
    dieUsage("$inputFile not found");
}

if (isset($args->args[1])) {
  $outputFile = $args->args[1];
  $path_parts = pathinfo($outputFile);
  $outputFileName = $path_parts['filename'].".".$path_parts['extension'];
  $sanitized_output = preg_replace('/[^0-9a-z\.\_\-]/i','',$outputFileName);
  if ($sanitized_output != $outputFileName) {
    dieUsage("Output filename problems. Try \"".$sanitized_output."\"");
  }
} else {
  $path_parts = pathinfo($inputFile);
  $outputFile = $path_parts['filename']."-booklet.".$path_parts['extension'];
} 

if (file_exists($outputFile)) {
    dieUsage("$outputFile already exist");
}

$scribusdoc = new DOMDocument();
error_reporting(0);
if ( !$scribusdoc->load($inputFile) ) {
   dieUsage("Unable to parse XML on $inputFile");
}
error_reporting(1);

if ($scribusdoc->documentElement->tagName != "SCRIBUSUTF8NEW") {
  dieUsage("$inputFile is not a valid Scribus file");
}
// To do: Check for scribus version in "SCRIBUSUTF8NEW" tag attribute "Version"

$scribusDocument = NULL;
foreach ($scribusdoc->documentElement->childNodes as $scribusElement){
    // Scan for DOCUMENT tag.   
    if ($scribusElement->tagName != "DOCUMENT") {continue; };
    $scribusDocument = $scribusElement;
    break;
}

$pages = $scribusDocument->getAttribute('ANZPAGES');
if (  ($pages % 4) != 0 ) {
    dieUsage("$inputFile has $pages pages and can't be placed on booklet");
}

$bookletPages = $pages / 2;
$scribusDocument->setAttribute( 'ANZPAGES', $bookletPages );
$pagewidth   = $scribusDocument->getAttribute('PAGEWIDTH');
$pageheight = $scribusDocument->getAttribute('PAGEHEIGHT');

$bookletheight = $pageheight;
$bookletwidth  = $pagewidth * 2;
$scribusDocument->setAttribute( 'PAGEWIDTH', num2str($bookletwidth) );
$scribusDocument->setAttribute( 'PAGEHEIGHT',num2str($bookletheight) );
$scribusDocument->setAttribute( 'ORIENTATION', 1 );

if ( $scribusDocument->getAttribute('BOOK') != 1 ) {
  dieUsage("$inputFile hasn't BOOK imposition");
}
$scribusDocument->setAttribute('BOOK', 0 );

$impositionSequence = array();
$i = 0;
$switchorder = FALSE;
while ( TRUE) {
    if ($switchorder ) {
      array_push($impositionSequence, $i);
      array_push($impositionSequence, $pages - 1 - $i);
    } else {
      array_push($impositionSequence, $pages - 1- $i);
      array_push($impositionSequence, $i);
    }
    $switchorder  = !$switchorder ;
    $i++;
    if ( ($pages - $i) <= $i ) { break; } ; 
}

$impositionTable = array();
$dummy = 0;
for ($i = 0; $i < count($impositionSequence) ; $i++) {
    $impositionTable[$impositionSequence[$i]]['bookletpage'] = $dummy;
    $impositionTable[$impositionSequence[$i]]['PAGEYPOS'] = 0;
    $i++;
    $impositionTable[$impositionSequence[$i]]['bookletpage'] = $dummy;
    $impositionTable[$impositionSequence[$i]]['PAGEYPOS'] = 0;
    $dummy++;
}

//To do: set ALWAYS correct page size.
$pagesize = $scribusDocument->getAttribute('PAGESIZE');
if ( $pagesize == "A4") {
  $scribusDocument->setAttribute( 'PAGESIZE', 'A3' );
}

$doccontrib = $scribusDocument->getAttribute('DOCCONTRIB');
$scribusDocument->setAttribute( 'DOCCONTRIB', $doccontrib . " booklet imposition made using booklettibus script version ". SCRIPT_VERSION . " " . RELEASE_DATE);

$scribusDocumentChilds = NULL;
if ( !$scribusDocument->hasChildNodes() ){
  dieUsage("$inputFile has DOCUMENT node without childs !?!?!?");
}

$firstPageY = 0;
$lastPageYoffset = 0;
$needtoremove = array();
$bookletypos = array();
foreach ($scribusDocument->childNodes as $scribusDocumentChild){ 
    switch ($scribusDocumentChild->tagName) {
    case "MASTERPAGE":
        $scribusDocumentChild->setAttribute( 'PAGEWIDTH', num2str($bookletwidth));
        $scribusDocumentChild->setAttribute( 'PAGEHEIGHT', num2str($bookletheight));
        $scribusDocumentChild->setAttribute( 'Size', 'A3' );
        $scribusDocumentChild->setAttribute( 'Orientation', '1' );
        break;
    case "PAGE":
        $scribusDocumentChild->setAttribute( 'PAGEWIDTH', num2str($bookletwidth));
        $scribusDocumentChild->setAttribute( 'PAGEHEIGHT', num2str($bookletheight));
        $scribusDocumentChild->setAttribute( 'Size', 'A3' );
        $scribusDocumentChild->setAttribute( 'Orientation', '1' );
        $tagPageNumber =  $scribusDocumentChild->getAttribute('NUM');
        $impositionTable[$tagPageNumber]['PAGEYPOS'] = $scribusDocumentChild->getAttribute('PAGEYPOS');
    
        //All PAGEY information collected.
        //Now place correct PAGEYPOS for booklet
        //I think that isn't the best way... It needs to be reviewed.
        if ( $tagPageNumber == ($pages - 1) ){
          for ( $i = 0 ; $i < $pages ; $i++) {
              if ( !iseven($i) ) { continue; };
              array_push($bookletypos, $impositionTable[$i]['PAGEYPOS'] );
          }
        }
        if ( ($tagPageNumber  + 1) > $bookletPages ) {
          array_push($needtoremove, $scribusDocumentChild);
        }
        break;
    case "PAGEOBJECT":
        $OwnPage = $scribusDocumentChild->getAttribute('OwnPage');
        $XPOS = $scribusDocumentChild->getAttribute('XPOS');
        $YPOS = $scribusDocumentChild->getAttribute('YPOS');
        
        $pageOffset = $impositionTable[$OwnPage]['PAGEYPOS']  - $bookletypos[$impositionTable[$OwnPage]['bookletpage']];

        $scribusDocumentChild->setAttribute('YPOS', num2str($YPOS - $pageOffset));
        $scribusDocumentChild->setAttribute('OwnPage', $impositionTable[$OwnPage]['bookletpage']);
        break;
     }
}

foreach ( $needtoremove as $toremove){
    $scribusDocument->removeChild($toremove);
}

$result =  $scribusdoc->save($outputFile);

if ( $result === FALSE ) {
    dieUsage("unable to write $outputFile");
}
echo "$result bytes written into $outputFile\r\n\r\n";

//EOF